"""Basic layout components.
"""
import grok
from hurry import yui
from megrok.layout import Page, Layout, EditForm, DisplayForm, AddForm
from waeup.interfaces import IWAeUPObject
from waeup.viewlets import LeftSidebar
from z3c.flashmessage.interfaces import IMessageSource
from zope.component import queryUtility, ComponentLookupError
from zope.interface import Interface
# Conditional import. getSite moved to zope.site.hooks in recent grok versions.
try:
    from zope.site.hooks import getSite
except ImportError:
    from zope.app.component.hooks import getSite

grok.templatedir('templates')

# The list of primary navigation tabs.
# Each entry is a tuple
#
#   (<id_number>, <order_number>, <text_to_render>, <link_target>)
#
# where id number is the number pages can use as their pnav attribute,
# order number determines the order in which the primary tabs are
# rendered, text_to_render gives the text displayed and link_target
# says, what url (relative to application URL) the tab should link to.
#
PRIMARY_NAV = [
    (0, 0, u'Home', ''),
    (1, 1, u'Faculties', '/faculties'),
    (2, 2, u'Search', '/search'),
    ]

class UtilityView(object):
    """A view mixin with useful methods.

    The ``pnav`` attribute (a number) tells, to which primary
    navigation tab a page declares to belong.
    """
    title = u'' # What appears in the content title...
    pnav = 0 # Primary navigation index...
    
    def application_url(self, name=None):
        """Return the URL of the nearest site.
        """
        site = getSite()
        #if not site:
        #    raise ComponentLookupError("No site found.")
        return self.url(site, name)

    def flash(self, message, type='message'):
        """Send a short message to the user.
        """
        source = queryUtility(IMessageSource, name='session')
        if source is None:
            return None
        source.send(message, type)
        return True

class WAeUPLayout(Layout, UtilityView):
    """A megrok.layout.Layout with additional methods.
    """
    grok.baseclass()
    
class WAeUPPage(Page, UtilityView):
    """A megrok.layout page with additional methods.
    """
    grok.baseclass()

class WAeUPDisplayFormPage(DisplayForm, UtilityView):
    """A megrok.layout.DisplayForm with additional methods.
    """
    grok.baseclass()

class WAeUPEditFormPage(EditForm, UtilityView):
    """A megrok.layout.EditForm with additional methods.
    """
    grok.baseclass()

class WAeUPAddFormPage(AddForm,  UtilityView):
    """A megrok.layout.AddForm with additional methods.
    """
    grok.baseclass()

class LeftSidebarLink(grok.Viewlet):
    """ An entry on left sidebar.

    This is only a baseclass that won't be rendered actually. Deriving
    viewlets can override certain values and will be rendered with the
    values set here as default.
    """
    grok.baseclass()
    grok.viewletmanager(LeftSidebar)
    grok.context(Interface)
    grok.order(5)
    grok.require('waeup.manageUniversity')

    title = 'Text of link'
    @property
    def url(self):
        return 'URL to link to'
    
    def render(self):
        return u'<div class="portlet"><a href="%s">%s</a></div>' % (
            self.url, self.title)

class SiteLayout(WAeUPLayout):
    """ The general site layout.
    """
    grok.context(IWAeUPObject)

    def getPrimaryNav(self, view):
        """Get an ordered list of primary navigation entries.

        Each is a dict with keys

          'url': <link-target>
          'active' : <value for HTML class attribute: 'active' or ''
          'title': <text to be rendered in menu tab>
          'order': <num indicating order num of this entry>

        Pages attatched to this layout can provide an attribute 'pnav'
        to indicate, to which tab they belong. This tab will then be
        rendered 'active'.
        """
        result = []
        app_url = view.application_url()
        for item in PRIMARY_NAV:
            active = ''
            pnav = getattr(view, 'pnav', 0)
            if pnav == item[0]:
                active = 'active'
            url =  app_url + item[3]
            result.append(
                dict(title=item[2],
                     url=url,
                     order=item[1],
                     active=active)
                )
        return sorted(result, key=lambda x: x['order'])
    
    def isAuthenticated(self):
        """Return True if the calling user is authenticated.
        """
        usertitle = self.request.principal.title
        return usertitle != 'Unauthenticated User'
    
    def getUserTitle(self):
        """Return principal title of current user.
        """
        usertitle = self.request.principal.title
        if usertitle == 'Unauthenticated User':
            return u''
        return usertitle

    def update(self):
        yui.reset_fonts_grids.need()
        yui.sam.need()
